{% extends "base.html" %}

{% block content %}
    <h1>Delete OAuth Token</h1>
    <p>Are you sure you want to delete the token for {{ object.provider }} ({{ object.name }})?</p>
    <form action="{% url 'oauth2_capture:revoke' slug=object.slug %}" method="post">
        {% csrf_token %}
        <input type="submit" value="Delete">
        <a href="{% url 'oauth2_capture:list' %}">Cancel</a>
    </form>
{% endblock %}
