# Generated by Django 5.1.1 on 2024-09-24 10:32

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("oauth2_capture", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="oauthtoken",
            name="access_token",
            field=models.TextField(help_text="The OAuth access token for API access."),
        ),
    ]
