from django.apps import AppConfig


class OAuth2CaptureConfig(AppConfig):
    """App configuration for the oauth2 capture app."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "oauth2_capture"
