{% extends "base.html" %}

{% block title %}OAuth2 Capture development site{% endblock %}

{% block content %}
    <h1>OAuth2 Capture development site</h1>
    <p>
    Use the menu above to do the things.
    </p>
{% endblock %}
